""" MainFrame of GrassBrowse Application.

Bernhard Reiter <bernhard@intevation.de>
Initially created: 16.1.2001 (experimented with Boa-Constructor)
"""
__version__="$Id: BrowseFrame.py,v 1.1 2001/01/20 21:05:35 bernhard Exp $"

from wxPython.wx import *
from wxPython.lib.filebrowsebutton import DirBrowseButton

import sys
from pylibgrass5 import *
import grassinfo

def create(parent,withcancelbutton=0):
    return wxFrame1(parent,withcancelbutton)

[wxID_LLISTBOX, wxID_MLISTBOX, wxID_CLISTBOX,
wxID_OKAYBUTTON, wxID_CANCELBUTTON ] =  \
        map(lambda _init_ctrls: wxNewId(), range(5))

class wxFrame1(wxFrame):
    def _init_ctrls(self, prnt, withcancelbutton): 
        wxFrame.__init__(self, size = wxSize(600, 455), id=-1,
            title = 'GRASS raster browser', 
            parent = prnt, 
            name = '', style = wxDEFAULT_FRAME_STYLE, pos = wxPoint(358, 203))


        self.dirBrowsePanel= DirBrowseButton(
            id=-1,
            parent=self, pos = wxPoint(20,20),
            size=wxSize(450,-1), changeCallback=self.OnDirChanged)


        self.llistBox = wxListBox(size = wxSize(60, 120), id=wxID_LLISTBOX,
            parent = self, name = 'Location')
        EVT_LISTBOX(self, wxID_LLISTBOX,self.OnllistBox)
        self.llistBoxdesc = wxStaticText(id=-1,label=self.llistBox.GetName(),
            parent=self, style=wxLB_SINGLE )

        self.mlistBox = wxListBox(size = wxSize(60, 120), id=wxID_MLISTBOX,
            style=wxLB_SINGLE,
            parent = self, name = 'Mapset' )
        EVT_LISTBOX(self, wxID_MLISTBOX,self.OnmlistBox)

        self.mlistBoxdesc = wxStaticText(id=-1,label=self.mlistBox.GetName(),
            style=wxLB_SINGLE,
            parent=self)

        self.clistBox = wxListBox(size = wxSize(20, 120), 
            style=wxLB_SINGLE,
            id=wxID_CLISTBOX, parent = self, name = 'Cell' )
        EVT_LISTBOX(self, wxID_CLISTBOX, self.OnclistBox)

        self.clistBoxdesc = wxStaticText(id=-1,label=self.clistBox.GetName(),
            parent=self)

        self.description = wxStaticText(label=":-)",
            id=-1,
            parent=self, size=wxSize(100,240))

        self.okaybutton= wxButton( id=wxID_OKAYBUTTON, parent=self,
            label="Okay", size=wxSize(100,20) )
        EVT_BUTTON(self, wxID_OKAYBUTTON, self.OnOkayButton)

        if withcancelbutton:
            self.cancelbutton =wxButton( id=wxID_CANCELBUTTON, parent=self,
                label="Cancel", size=wxSize(100,20))
            EVT_BUTTON(self, wxID_CANCELBUTTON, self.OnCancelButton)
        else:
            self.okaybutton.SetLabel("Exit")


        # set up layout system
        #
        lsizer=wxBoxSizer(wxVERTICAL)
        lsizer.Add(self.llistBoxdesc,0,wxALIGN_LEFT | wxALL, 2 )
        lsizer.Add(self.llistBox, 1, wxEXPAND | wxALL, 1)

        msizer=wxBoxSizer(wxVERTICAL)
        msizer.Add(self.mlistBoxdesc,0,wxALIGN_LEFT | wxALL, 2 )
        msizer.Add(self.mlistBox, 1, wxEXPAND | wxALL, 1)

        csizer=wxBoxSizer(wxVERTICAL)
        csizer.Add(self.clistBoxdesc,0,wxALIGN_LEFT | wxALL, 2 )
        csizer.Add(self.clistBox, 1, wxEXPAND | wxALL, 1)

        sbox=wxStaticBox(id=-1,parent=self,label="Info")
        dsizer=wxStaticBoxSizer(sbox,wxVERTICAL)
        dsizer.Add(self.description, 1, wxEXPAND | wxALL, 1)

        self.listsizer=wxBoxSizer(wxHORIZONTAL)
        self.listsizer.Add(lsizer, 1, wxEXPAND | wxALL, 5)
        self.listsizer.Add(msizer, 1, wxEXPAND | wxALL, 5)
        self.listsizer.Add(csizer, 1, wxEXPAND | wxALL, 5)
        self.listsizer.Add(dsizer, 3, wxEXPAND | wxALL, 8)

        self.buttonsizer=wxBoxSizer(wxHORIZONTAL)
        if withcancelbutton:
            self.buttonsizer.Add(self.cancelbutton,1, wxALL , 10 )
        self.buttonsizer.Add(self.okaybutton, 1, wxALL, 10 )

        self.topsizer=wxBoxSizer(wxVERTICAL)
        self.topsizer.Add(self.dirBrowsePanel, 0, wxALL, 10)
        self.topsizer.Add(self.listsizer, 1, wxEXPAND)
        self.topsizer.Add(self.buttonsizer, 0, wxALIGN_RIGHT)

        self.SetAutoLayout(1)
        self.SetSizer(self.topsizer)
        self.topsizer.Fit(self)
        self.topsizer.SetSizeHints(self)


    def __init__(self, parent, withcancelbutton=0): 
        # instance variables
        self.progname=""
        self.preselected=None
        self.in_startup=1
        self.okay=0
        self.gisdbase=None
        self.location=None
        self.mapset=None
        self.cellname=None
        self.withcancelbutton=withcancelbutton
        # construct gui elements
        self._init_ctrls(parent, withcancelbutton)
        # make sure that we initilise the content
        self.gisdbasechanged(None)

#******************************************************************************
#* Event Handling
#*
    def OnDirChanged(self,evt):
        """ Handle changeCallback event from DirBrowseButton. 
        
        Also handle bug in DirBrowseButton in wxPython-2.2.2,
        that generates an event when the constructior is not completed.
        """
        # dirty check if the constructor was completed and we have
        # the instance variable llistBox
        if self.__dict__.has_key("llistBox"):
            d=evt.GetString()
            print "selected database: '%s'" % d 
            if d =="": d=None
            self.gisdbasechanged(d)

    def OnllistBox(self,evt):
        self.location_changed()

    def OnmlistBox(self,evt):
        self.mapset_changed()

    def OnclistBox(self,evt):
        self.cell_changed()

    def OnOkayButton(self,evt):
        self.okay=1
        self.Close()
        
    def OnCancelButton(self,evt):
        self.okay=0
        self.Close()
        

#******************************************************************************
#* "Normal" Methods
#*


    def gisdbasechanged(self,gisdbase_new):
        """ Make changes if we deal with a new database."""
        self.llistBox.Clear()
        self.gisdbase=None
        self.mlistBox.Clear()
        self.location=None
        self.clistBox.Clear()
        self.mapset=None
        self.description.SetLabel("")
        self.cellname=None

        self.gisdbase=gisdbase_new
        state=G_gisinit_2(self.progname,self.gisdbase)
        if state==0 and self.preselected==None and self.in_startup:
            # we have a preselected combination
            self.preselected=grassinfo.currentposition()
            self.dirBrowsePanel.SetValue(self.preselected[0])
            self.in_startup=0
            return

        locations=G_fetch_list_locations(gisdbase_new)
        for location in locations:
            self.llistBox.Append(location)

        if self.preselected:
            i=self.llistBox.FindString(self.preselected[1])
            self.llistBox.SetSelection(i)
            self.location_changed()
            self.preselected=None
        else:
            self.location_changed()

    def location_changed(self):
        " Fill mapsetlist for a new location. "
        self.mlistBox.Clear()
        self.location=None
        self.clistBox.Clear()
        self.mapset=None
        self.description.SetLabel("")
        self.cellname=None

        self.location=self.llistBox.GetStringSelection()
        status=G_gisinit_2(self.progname,self.dirBrowsePanel.GetValue(), 
                                self.location)
        mapsets=G_fetch_list_mapsets(None)
        for mapset in mapsets:
            self.mlistBox.Append(mapset)

        if self.preselected:
            i=self.mlistBox.FindString(self.preselected[2])
            self.mlistBox.SetSelection(i)
        self.mapset_changed()

    def mapset_changed(self):
        " Fill cell and image group list if the mapset changed. "
        self.clistBox.Clear()
        self.mapset=None
        self.description.SetLabel("")
        self.cellname=None

        self.mapset=self.mlistBox.GetStringSelection()
        print "new mapset: ", self.mapset

        status=G_gisinit_2(self.progname,self.dirBrowsePanel.GetValue(),
                    self.llistBox.GetStringSelection(), self.mapset)
        cellhds=G_fetch_list_element("cellhd","")
        groups=G_fetch_list_element("group", "")
#        print status, cellhds, groups
        for cellhd in cellhds:
            self.clistBox.Append(cellhd)
        for group in groups:
            # FIXME somehow indicate imagery group, without corrupting name
            self.clistBox.Append(group)

        self.cell_changed()

    def cell_changed(self):
        """ Give out info for the cell or imagery group. """
        self.description.SetLabel("")
        self.cellname=None

        self.cellname=self.clistBox.GetStringSelection()
        if self.cellname != "":
            #check for group
            status,mapset,group_name=I_check_group(self.cellname)
            s=""
            if status:
                cellnames=grassinfo.get_imagerygrouplist(group_name)
                s="Imagery Group '%s' with %d raster maps in this mapset:\n" %\
                        (self.cellname, len(cellnames))
                for name in cellnames:
                    s=s+"   %s\n" % name
            else:
                cell_header=Cell_head()
                s=grassinfo.get_cellinfo(self.cellname, cell_header)

            self.description.SetLabel(s)
            self.okaybutton.Enable(1)
        else:
            if self.withcancelbutton:
                self.okaybutton.Enable(0)
